#include "Keyboard.h"
#define DEBUG
#define PIN_WIRE_SDA D3
#define PIN_WIRE_SCL D4
Keyboard::Keyboard(uint8_t _debounce)
{
    this->_debounce = _debounce;
}
void Keyboard::begin(TwoWire *databus)
{
    pcf8574 = new PCF8574(databus, 0x21, PIN_WIRE_SDA, PIN_WIRE_SCL);
    pcf8574->pinMode(0, OUTPUT);
    for (int i = 1; i < 8; i++)
    {
        if (i < 5)
            pcf8574->pinMode(i, INPUT_PULLUP);
        else
            pcf8574->pinMode(i, OUTPUT);
    }

    Serial.print("Init pcf8574...");
    if (pcf8574->begin())
        Serial.println("OK");
    else
        Serial.println("KO");

    pcf8574->digitalWrite(1, HIGH);
    for (int i = 5; i < 8; i++)
        pcf8574->digitalWrite(i, HIGH);
}
Keyboard::~Keyboard()
{
    if (pcf8574 != nullptr)
        delete pcf8574;
}
void Keyboard::scan()
{
    if (millis() < _timeElapsed + this->_debounce)
        return;
    uint8_t key = 0;
    for (int i = 7; i > 4; i--) // Columns
    {
        pcf8574->digitalWrite(i, LOW);
        delay(15);
        for (int j = 1; j < 5; j++)
        { // Rows
            key++;
            uint8_t val = pcf8574->digitalRead(j);
#ifdef DEBUG
            Serial.println("i=" + String(i) + " j=" + String(j) + " v=" + String(val));
            delay(500);
#endif
            if (val == 0)
            {
                _timeElapsed = millis();
                this->_lastKey = key;
                this->_buffer.push_back(mapChr(key));
                pcf8574->digitalWrite(i, HIGH);
                return;
            }
        }
        pcf8574->digitalWrite(i, HIGH);
    }
}
void Keyboard::scanAsync()
{
    /** Without delay - scanning only one column per cycle **/
    if (millis() < _timeElapsed + this->_debounce)
        return;
    uint8_t key = 0;
    switch (_current_scan_col)
    {
        default:
            pcf8574->digitalWrite(7, LOW);
            _current_scan_col = 1;
            break;
        case 1:
        {
            scanColumn(&key, 1, 4);
            pcf8574->digitalWrite(6, LOW);            
            pcf8574->digitalWrite(7, HIGH);
            _current_scan_col = 2;
            break;
        }
        case 2:
        {
            key=4;
            scanColumn(&key, 1, 4);
            pcf8574->digitalWrite(5, LOW);            
            pcf8574->digitalWrite(6, HIGH);
            _current_scan_col = 3;
            break;
        }
        case 3:
        {
            key=8;
            scanColumn(&key, 1, 4);
            pcf8574->digitalWrite(7, LOW);            
            pcf8574->digitalWrite(5, HIGH);
            _current_scan_col = 1;
            break;
        }
        break;
    }
}

void Keyboard::scanColumn(uint8_t* key_ptr, uint8_t start, uint8_t stop){
    for (int j = start; j <= stop; j++)
            { // Rows
                (*key_ptr)++;
                uint8_t val = pcf8574->digitalRead(j);
                if (val == 0)
                {
                    _timeElapsed = millis();
                    this->_lastKey = *key_ptr;
                    this->_buffer.push_back(mapChr(*key_ptr));
                    Serial.print(*key_ptr);
                    return;
                }
            }
}
char Keyboard::mapChr(uint8_t key)
{

    switch (key)
    {
    case 1:
        return '1';
    case 2:
        return '4';
    case 3:
        return '7';
    case 4:
        return 'O';
    case 5:
        return '2';
    case 6:
        return '5';
    case 7:
        return '8';
    case 8:
        return '0';
    case 9:
        return '3';
    case 10:
        return '6';
    case 11:
        return '9';
    case 12:
        return 'X';

    default:
        return 'F';
    }
}
char Keyboard::getLastChr()
{
    return this->mapChr(this->_lastKey);
}

bool Keyboard::available()
{
    return (!this->_buffer.empty());
}
void Keyboard::clear()
{
    this->_buffer.clear();
}
String Keyboard::getString()
{
    String out;
    for (auto &&i : this->_buffer)
    {
        out.concat(i);
    }
    this->_buffer.clear();
    return out;
}