
#include <Arduino.h>
#include <Wire.h>
#include "Keyboard.h"
#include <SPI.h>
#include <MFRC522.h>

// RFID Reader
#define SS_PIN D8
#define RST_PIN D1
#define RFID_TIMEOUT 3000
MFRC522 mfrc522(SS_PIN); // Create MFRC522 instance
MFRC522::MIFARE_Key key;
String rfid = "";
String lastRfid = "";
unsigned long lastRfidScan = 0;
// i2C Bus
#define PIN_WIRE_SDA D3
#define PIN_WIRE_SCL D4
#include <LiquidCrystal_I2C.h>
Keyboard keyboard(200);
LiquidCrystal_I2C lcd(0x27, 20, 4);
int display_state = 0;
String pin;
unsigned long lastDisplayUpdate = 0;
unsigned long displayTimer1 = 0;
bool displayUpdate = true;
void setup()
{
	Serial.begin(9600);
	Serial.print("Starting");
	delay(500);
	keyboard.begin(&Wire);

	// LCD
	lcd.init();
	lcd.backlight();

	// RFID
	SPI.begin();
	SPI.setClockDivider(SPI_CLOCK_DIV8);
	mfrc522.PCD_Init();
	mfrc522.PCD_SetAntennaGain(mfrc522.RxGain_max);
}

namespace states
{
	enum screenstates
	{
		MAIN,
		ENTER_PIN_START,
		ENTER_PIN_ADD_NUM,
		VALIDATE_PIN,
		ABORT,
		DELAY,
		READ_RFID

	};
}
void loop()
{
	keyboard.scanAsync();

	if (mfrc522.PICC_IsNewCardPresent() && mfrc522.PICC_ReadCardSerial())
	{
		rfid = "";
		for (byte i = 0; i < mfrc522.uid.size; i++)
		{
			rfid += mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " ";
			rfid += String(mfrc522.uid.uidByte[i], HEX);
		}
		rfid.trim();
		rfid.toUpperCase();
		if (rfid != lastRfid || millis() > lastRfidScan + RFID_TIMEOUT)
		{

			display_state = states::READ_RFID;
			displayUpdate = true;
			Serial.print(rfid);
			lastRfid = rfid;
			lastRfidScan = millis();
		}
	}
	if (keyboard.available())
	{
		if (keyboard.getLastChr() == 'X')
		{
			keyboard.clear();
			display_state = states::ABORT;
			displayUpdate = true;
		}
		else if (('0' <= keyboard.getLastChr()) && (keyboard.getLastChr() <= '9'))
		{
			Serial.print(keyboard.getLastChr());
			if (display_state == states::ENTER_PIN_START)
				display_state = states::ENTER_PIN_ADD_NUM;
			else if (display_state != states::ENTER_PIN_ADD_NUM)
				display_state = states::ENTER_PIN_START;
			displayUpdate = true;
		}
		else if (keyboard.getLastChr() == 'O' && (display_state == 2 || display_state == 1))
		{
			keyboard.clear();
			display_state = states::VALIDATE_PIN;
			displayUpdate = true;
		}
	}
	if (displayUpdate)
	{
		switch (display_state)
		{
		case states::MAIN:
			lcd.setCursor(0, 0);
			lcd.print("Welcome!");
			lcd.setCursor(0, 1);
			lcd.print("Enter Pin / Card");
			displayUpdate = false;
			break;
		case states::ENTER_PIN_START: // Starting Pin
			pin.clear();
			lcd.clear();
			lcd.print("Please enter Pin");
			lcd.setCursor(0, 1);
			pin.concat(keyboard.getString());
			lcd.print("Pin: " + pin);
			display_state = 2;
			displayUpdate = false;
			break;
		case states::ENTER_PIN_ADD_NUM:
		{
			String input = keyboard.getString();
			pin.concat(input);
			lcd.print(input);
			displayUpdate = false;
			break;
		}
		case states::VALIDATE_PIN:
			lcd.clear();
			lcd.setCursor(0, 0);
			lcd.print("Validating PIN");
			lcd.setCursor(0, 1);
			if (pin == "2626")
				lcd.print("Accepted!");
			else
				lcd.print("Acess denied!");
			pin.clear();
			displayTimer1 = millis() + 3000;
			display_state = states::DELAY;
			break;
		case states::ABORT:
			lcd.clear();
			lcd.print("Input aborted!");
			displayTimer1 = millis() + 3000;
			display_state = states::DELAY;
			break;
		case states::DELAY: // Delay
			if (displayTimer1 != 0 && displayTimer1 < millis())
			{
				lcd.clear();
				display_state = states::MAIN;
			}
			break;
		case states::READ_RFID:
		{
			lcd.clear();
			lcd.print("Card detected.");
			lcd.setCursor(0, 1);

			lcd.print(rfid);
			displayTimer1 = millis() + 3000;
			display_state = states::DELAY;
			break;
		}
		default:
			break;
		}
	}
	//String input = keyboard.getString();
	//Serial.println("Key " + input +  "was pressed");
}